@@ -51,6 +51,9 @@
 // Workaround in case INFINITY is not defined in math.h or if the OS is CentOS
 #define OJ_INFINITY (1.0/0.0)
 
+// Extra padding at end of buffer.
+#define BUFFER_EXTRA 10
+
 typedef unsigned long	ulong;
 
 static void	raise_strict(VALUE obj);
@@ -228,11 +231,11 @@ grow(Out out, size_t len) {
 	size += len;
     }
     if (out->allocated) {
-	buf = REALLOC_N(out->buf, char, (size + 10));
+	buf = REALLOC_N(out->buf, char, (size + BUFFER_EXTRA));
     } else {
-	buf = ALLOC_N(char, (size + 10));
+	buf = ALLOC_N(char, (size + BUFFER_EXTRA));
 	out->allocated = 1;
-	memcpy(buf, out->buf, out->end - out->buf);
+	memcpy(buf, out->buf, out->end - out->buf + BUFFER_EXTRA);
     }
     if (0 == buf) {
 	rb_raise(rb_eNoMemError, "Failed to create string. [%d:%s]\n", ENOSPC, strerror(ENOSPC));
@@ -488,8 +491,8 @@ dump_cstr(const char *str, size_t cnt, int is_sym, int escape1, Out out) {
 	cmap = hibit_friendly_chars;
 	size = hibit_friendly_size((uint8_t*)str, cnt);
     }
-    if (out->end - out->cur <= (long)size + 10) { // extra 10 for escaped first char, quotes, and sym
-	grow(out, size + 10);
+    if (out->end - out->cur <= (long)size + BUFFER_EXTRA) { // extra 10 for escaped first char, quotes, and sym
+	grow(out, size + BUFFER_EXTRA);
     }
     *out->cur++ = '"';
     if (escape1) {
@@ -1717,7 +1720,7 @@ void
 oj_dump_obj_to_json(VALUE obj, Options copts, Out out) {
     if (0 == out->buf) {
 	out->buf = ALLOC_N(char, 4096);
-	out->end = out->buf + 4085; // 1 less than end plus extra for possible errors
+	out->end = out->buf + 4095 - BUFFER_EXTRA; // 1 less than end plus extra for possible errors
 	out->allocated = 1;
     }
     out->cur = out->buf;
@@ -1746,7 +1749,7 @@ oj_write_obj_to_file(VALUE obj, const char *path, Options copts) {
     int		ok;
 
     out.buf = buf;
-    out.end = buf + sizeof(buf) - 10;
+    out.end = buf + sizeof(buf) - BUFFER_EXTRA;
     out.allocated = 0;
     oj_dump_obj_to_json(obj, copts, &out);
     size = out.cur - out.buf;
@@ -1774,16 +1777,17 @@ oj_write_obj_to_stream(VALUE obj, VALUE stream, Options copts) {
     struct _Out out;
     ssize_t	size;
     VALUE	clas = rb_obj_class(stream);
+#if !IS_WINDOWS
     VALUE	s;
+#endif
 
     out.buf = buf;
-    out.end = buf + sizeof(buf) - 10;
+    out.end = buf + sizeof(buf) - BUFFER_EXTRA;
     out.allocated = 0;
     oj_dump_obj_to_json(obj, copts, &out);
     size = out.cur - out.buf;
     if (oj_stringio_class == clas) {
 	rb_funcall(stream, oj_write_id, 1, rb_str_new(out.buf, size));
-#ifndef JRUBY_RUBY
 #if !IS_WINDOWS
     } else if (rb_respond_to(stream, oj_fileno_id) && Qnil != (s = rb_funcall(stream, oj_fileno_id, 0))) {
 	int	fd = FIX2INT(s);
@@ -1794,7 +1798,6 @@ oj_write_obj_to_stream(VALUE obj, VALUE stream, Options copts) {
 	    }
 	    rb_raise(rb_eIOError, "Write failed. [%d:%s]\n", errno, strerror(errno));
 	}
-#endif
 #endif
     } else if (rb_respond_to(stream, oj_write_id)) {
 	rb_funcall(stream, oj_write_id, 1, rb_str_new(out.buf, size));
@@ -1988,7 +1991,7 @@ void
 oj_dump_leaf_to_json(Leaf leaf, Options copts, Out out) {
     if (0 == out->buf) {
 	out->buf = ALLOC_N(char, 4096);
-	out->end = out->buf + 4085; // 1 less than end plus extra for possible errors
+	out->end = out->buf + 4095 - BUFFER_EXTRA; // 1 less than end plus extra for possible errors
 	out->allocated = 1;
     }
     out->cur = out->buf;
@@ -2007,7 +2010,7 @@ oj_write_leaf_to_file(Leaf leaf, const char *path, Options copts) {
     FILE	*f;
 
     out.buf = buf;
-    out.end = buf + sizeof(buf) - 10;
+    out.end = buf + sizeof(buf) - BUFFER_EXTRA;
     out.allocated = 0;
     oj_dump_leaf_to_json(leaf, copts, &out);
     size = out.cur - out.buf;
@@ -2067,15 +2070,17 @@ maybe_comma(StrWriter sw) {
 
 void
 oj_str_writer_push_object(StrWriter sw, const char *key) {
-    size_t	size;
+    long	size;
 
     key_check(sw, key);
     size = sw->depth * sw->out.indent + 3;
-    if (sw->out.end - sw->out.cur <= size) {
+    if (sw->out.end - sw->out.cur <= (long)size) {
 	grow(&sw->out, size);
     }
     maybe_comma(sw);
-    fill_indent(&sw->out, sw->depth);
+    if (0 < sw->depth) {
+	fill_indent(&sw->out, sw->depth);
+    }
     if (0 != key) {
 	dump_cstr(key, strlen(key), 0, 0, &sw->out);
 	*sw->out.cur++ = ':';
@@ -2086,7 +2091,7 @@ oj_str_writer_push_object(StrWriter sw, const char *key) {
 
 void
 oj_str_writer_push_array(StrWriter sw, const char *key) {
-    size_t	size;
+    long	size;
 
     key_check(sw, key);
     size = sw->depth * sw->out.indent + 3;
@@ -2094,6 +2099,9 @@ oj_str_writer_push_array(StrWriter sw, const char *key) {
 	grow(&sw->out, size);
     }
     maybe_comma(sw);
+    if (0 < sw->depth) {
+	fill_indent(&sw->out, sw->depth);
+    }
     if (0 != key) {
 	dump_cstr(key, strlen(key), 0, 0, &sw->out);
 	*sw->out.cur++ = ':';
@@ -2104,7 +2112,7 @@ oj_str_writer_push_array(StrWriter sw, const char *key) {
 
 void
 oj_str_writer_push_value(StrWriter sw, VALUE val, const char *key) {
-    size_t	size;
+    long	size;
 
     key_check(sw, key);
     size = sw->depth * sw->out.indent + 3;
@@ -2112,6 +2120,9 @@ oj_str_writer_push_value(StrWriter sw, VALUE val, const char *key) {
 	grow(&sw->out, size);
     }
     maybe_comma(sw);
+    if (0 < sw->depth) {
+	fill_indent(&sw->out, sw->depth);
+    }
     if (0 != key) {
 	dump_cstr(key, strlen(key), 0, 0, &sw->out);
 	*sw->out.cur++ = ':';
@@ -2119,9 +2130,29 @@ oj_str_writer_push_value(StrWriter sw, VALUE val, const char *key) {
     dump_val(val, sw->depth, &sw->out);
 }
 
+void
+oj_str_writer_push_json(StrWriter sw, const char *json, const char *key) {
+    long	size;
+
+    key_check(sw, key);
+    size = sw->depth * sw->out.indent + 3;
+    if (sw->out.end - sw->out.cur <= size) {
+	grow(&sw->out, size);
+    }
+    maybe_comma(sw);
+    if (0 < sw->depth) {
+	fill_indent(&sw->out, sw->depth);
+    }
+    if (0 != key) {
+	dump_cstr(key, strlen(key), 0, 0, &sw->out);
+	*sw->out.cur++ = ':';
+    }
+    dump_raw(json, strlen(json), &sw->out);
+}
+
 void
 oj_str_writer_pop(StrWriter sw) {
-    size_t	size;
+    long	size;
     DumpType	type = sw->types[sw->depth];
 
     sw->depth--;
@@ -2143,6 +2174,9 @@ oj_str_writer_pop(StrWriter sw) {
 	*sw->out.cur++ = ']';
 	break;
     }
+    if (0 == sw->depth && 0 < sw->out.indent) {
+	*sw->out.cur++ = '\n';
+    }
 }
 
 void
