@@ -38,6 +38,7 @@
 #include <errno.h>
 
 #include "oj.h"
+#include "encode.h"
 
 // maximum to allocate on the stack, arbitrary limit
 #define SMALL_XML	65536
@@ -266,9 +267,7 @@ leaf_value(Doc doc, Leaf leaf) {
 	    break;
 	case T_STRING:
 	    leaf->value = rb_str_new2(leaf->str);
-#if HAS_ENCODING_SUPPORT
-	    rb_enc_associate(leaf->value, oj_utf8_encoding);
-#endif
+	    leaf->value = oj_encode(leaf->value);
 	    leaf->value_type = RUBY_VAL;
 	    break;
 	case T_ARRAY:
@@ -467,9 +466,7 @@ leaf_hash_value(Doc doc, Leaf leaf) {
 
 	do {
 	    key = rb_str_new2(e->key);
-#if HAS_ENCODING_SUPPORT
-	    rb_enc_associate(key, oj_utf8_encoding);
-#endif
+	    key = oj_encode(key);
 	    rb_hash_aset(h, key, leaf_value(doc, e));
 	    e = e->next;
 	} while (e != first);
@@ -902,6 +899,9 @@ get_doc_leaf(Doc doc, const char *path) {
 	} else {
 	    size_t	cnt = doc->where - doc->where_path;
 
+	    if (MAX_STACK <= cnt) {
+		rb_raise(rb_const_get_at(Oj, rb_intern("DepthError")), "Path too deep. Limit is %d levels.", MAX_STACK);
+	    }
 	    memcpy(stack, doc->where_path, sizeof(Leaf) * cnt);
 	    lp = stack + cnt;
 	}
@@ -988,11 +988,15 @@ each_leaf(Doc doc, VALUE self) {
 	    Leaf	e = first;
 
 	    doc->where++;
+	    if (MAX_STACK <= doc->where - doc->where_path) {
+		rb_raise(rb_const_get_at(Oj, rb_intern("DepthError")), "Path too deep. Limit is %d levels.", MAX_STACK);
+	    }
 	    do {
 		*doc->where = e;
 		each_leaf(doc, self);
 		e = e->next;
 	    } while (e != first);
+	    doc->where--;
 	}
     } else {
 	rb_yield(self);
@@ -1268,9 +1272,7 @@ doc_local_key(VALUE self) {
 
     if (T_HASH == leaf->parent_type) {
 	key = rb_str_new2(leaf->key);
-#if HAS_ENCODING_SUPPORT
-	rb_enc_associate(key, oj_utf8_encoding);
-#endif
+	key = oj_encode(key);
     } else if (T_ARRAY == leaf->parent_type) {
 	key = LONG2NUM(leaf->index);
     }
