@@ -866,6 +866,7 @@ str_writer_init(StrWriter sw) {
     sw->out.hash_cnt = 0;
     sw->out.opts = &sw->opts;
     sw->out.indent = sw->opts.indent;
+    sw->out.depth = 0;
 }
 
 /* call-seq: new(options)
@@ -881,6 +882,8 @@ str_writer_new(int argc, VALUE *argv, VALUE self) {
     if (1 == argc) {
 	oj_parse_options(argv[0], &sw->opts);
     }
+    sw->out.indent = sw->opts.indent;
+
     return Data_Wrap_Struct(oj_string_writer_class, 0, str_writer_free, sw);
 }
 
@@ -892,22 +895,28 @@ str_writer_new(int argc, VALUE *argv, VALUE self) {
  */
 static VALUE
 str_writer_push_object(int argc, VALUE *argv, VALUE self) {
+    StrWriter	sw = (StrWriter)DATA_PTR(self);
+
     switch (argc) {
     case 0:
-	oj_str_writer_push_object((StrWriter)DATA_PTR(self), 0);
+	oj_str_writer_push_object(sw, 0);
 	break;
     case 1:
 	if (Qnil == argv[0]) {
-	    oj_str_writer_push_object((StrWriter)DATA_PTR(self), 0);
+	    oj_str_writer_push_object(sw, 0);
 	} else {
 	    rb_check_type(argv[0], T_STRING);
-	    oj_str_writer_push_object((StrWriter)DATA_PTR(self), StringValuePtr(argv[0]));
+	    oj_str_writer_push_object(sw, StringValuePtr(argv[0]));
 	}
 	break;
     default:
 	rb_raise(rb_eArgError, "Wrong number of argument to 'push_object'.");
 	break;
     }
+    if (rb_block_given_p()) {
+	rb_yield(Qnil);
+	oj_str_writer_pop(sw);
+    }
     return Qnil;
 }
 
@@ -919,22 +928,28 @@ str_writer_push_object(int argc, VALUE *argv, VALUE self) {
  */
 static VALUE
 str_writer_push_array(int argc, VALUE *argv, VALUE self) {
+    StrWriter	sw = (StrWriter)DATA_PTR(self);
+
     switch (argc) {
     case 0:
-	oj_str_writer_push_array((StrWriter)DATA_PTR(self), 0);
+	oj_str_writer_push_array(sw, 0);
 	break;
     case 1:
 	if (Qnil == argv[0]) {
-	    oj_str_writer_push_array((StrWriter)DATA_PTR(self), 0);
+	    oj_str_writer_push_array(sw, 0);
 	} else {
 	    rb_check_type(argv[0], T_STRING);
-	    oj_str_writer_push_array((StrWriter)DATA_PTR(self), StringValuePtr(argv[0]));
+	    oj_str_writer_push_array(sw, StringValuePtr(argv[0]));
 	}
 	break;
     default:
 	rb_raise(rb_eArgError, "Wrong number of argument to 'push_object'.");
 	break;
     }
+    if (rb_block_given_p()) {
+	rb_yield(Qnil);
+	oj_str_writer_pop(sw);
+    }
     return Qnil;
 }
 
@@ -965,6 +980,36 @@ str_writer_push_value(int argc, VALUE *argv, VALUE self) {
     return Qnil;
 }
 
+/* call-seq: push_json(value, key=nil)
+ *
+ * Pushes a string onto the JSON document. The String must be a valid JSON
+ * encoded string. No additional checking is done to verify the validity of the
+ * string.
+ * @param [String] value JSON document to add to the JSON document
+ * @param [String] key the key if adding to an object in the JSON document
+ */
+static VALUE
+str_writer_push_json(int argc, VALUE *argv, VALUE self) {
+    rb_check_type(argv[0], T_STRING);
+    switch (argc) {
+    case 1:
+	oj_str_writer_push_json((StrWriter)DATA_PTR(self), StringValuePtr(*argv), 0);
+	break;
+    case 2:
+	if (Qnil == argv[1]) {
+	    oj_str_writer_push_json((StrWriter)DATA_PTR(self), StringValuePtr(*argv), 0);
+	} else {
+	    rb_check_type(argv[1], T_STRING);
+	    oj_str_writer_push_json((StrWriter)DATA_PTR(self), StringValuePtr(*argv), StringValuePtr(argv[1]));
+	}
+	break;
+    default:
+	rb_raise(rb_eArgError, "Wrong number of argument to 'push_json'.");
+	break;
+    }
+    return Qnil;
+}
+
 /* call-seq: pop()
  *
  * Pops up a level in the JSON document closing the array or object that is
@@ -1074,17 +1119,17 @@ stream_writer_new(int argc, VALUE *argv, VALUE self) {
     int			fd = 0;
     VALUE		stream = argv[0];
     VALUE		clas = rb_obj_class(stream);
-    VALUE		s;
     StreamWriter	sw;
+#if !IS_WINDOWS
+    VALUE		s;
+#endif
     
     if (oj_stringio_class == clas) {
 	type = STRING_IO;
-#ifndef JRUBY_RUBY
 #if !IS_WINDOWS
     } else if (rb_respond_to(stream, oj_fileno_id) && Qnil != (s = rb_funcall(stream, oj_fileno_id, 0))) {
 	type = FILE_IO;
 	fd = FIX2INT(s);
-#endif
 #endif
     } else if (rb_respond_to(stream, oj_write_id)) {
 	type = STREAM_IO;
@@ -1196,6 +1241,40 @@ stream_writer_push_value(int argc, VALUE *argv, VALUE self) {
     return Qnil;
 }
 
+/* call-seq: push_value(value, key=nil)
+ *
+ * Pushes a string onto the JSON document. The String must be a valid JSON
+ * encoded string. No additional checking is done to verify the validity of the
+ * string.
+ * @param [Object] value value to add to the JSON document
+ * @param [String] key the key if adding to an object in the JSON document
+ */
+static VALUE
+stream_writer_push_json(int argc, VALUE *argv, VALUE self) {
+    StreamWriter	sw = (StreamWriter)DATA_PTR(self);
+
+    rb_check_type(argv[0], T_STRING);
+    stream_writer_reset_buf(sw);
+    switch (argc) {
+    case 1:
+	oj_str_writer_push_json((StrWriter)DATA_PTR(self), StringValuePtr(*argv), 0);
+	break;
+    case 2:
+	if (Qnil == argv[0]) {
+	    oj_str_writer_push_json((StrWriter)DATA_PTR(self), StringValuePtr(*argv), 0);
+	} else {
+	    rb_check_type(argv[1], T_STRING);
+	    oj_str_writer_push_json((StrWriter)DATA_PTR(self), StringValuePtr(*argv), StringValuePtr(argv[1]));
+	}
+	break;
+    default:
+	rb_raise(rb_eArgError, "Wrong number of argument to 'push_json'.");
+	break;
+    }
+    stream_writer_write(sw);
+    return Qnil;
+}
+
 /* call-seq: pop()
  *
  * Pops up a level in the JSON document closing the array or object that is
@@ -1578,6 +1657,7 @@ define_mimic_json(int argc, VALUE *argv, VALUE self) {
     return mimic;
 }
 
+/*
 extern void	oj_hash_test();
 
 static VALUE
@@ -1585,6 +1665,7 @@ hash_test(VALUE self) {
     oj_hash_test();
     return Qnil;
 }
+*/
 
 #if !HAS_ENCODING_SUPPORT
 static VALUE
@@ -1621,6 +1702,7 @@ void Init_oj() {
     rb_define_method(oj_string_writer_class, "push_object", str_writer_push_object, -1);
     rb_define_method(oj_string_writer_class, "push_array", str_writer_push_array, -1);
     rb_define_method(oj_string_writer_class, "push_value", str_writer_push_value, -1);
+    rb_define_method(oj_string_writer_class, "push_json", str_writer_push_json, -1);
     rb_define_method(oj_string_writer_class, "pop", str_writer_pop, 0);
     rb_define_method(oj_string_writer_class, "pop_all", str_writer_pop_all, 0);
     rb_define_method(oj_string_writer_class, "reset", str_writer_reset, 0);
@@ -1631,6 +1713,7 @@ void Init_oj() {
     rb_define_method(oj_stream_writer_class, "push_object", stream_writer_push_object, -1);
     rb_define_method(oj_stream_writer_class, "push_array", stream_writer_push_array, -1);
     rb_define_method(oj_stream_writer_class, "push_value", stream_writer_push_value, -1);
+    rb_define_method(oj_stream_writer_class, "push_json", stream_writer_push_json, -1);
     rb_define_method(oj_stream_writer_class, "pop", stream_writer_pop, 0);
     rb_define_method(oj_stream_writer_class, "pop_all", stream_writer_pop_all, 0);
 
@@ -1650,7 +1733,7 @@ void Init_oj() {
     oj_utf8_encoding = Qnil;
 #endif
 
-    rb_define_module_function(Oj, "hash_test", hash_test, 0);
+    //rb_define_module_function(Oj, "hash_test", hash_test, 0);
 
     rb_define_module_function(Oj, "default_options", get_def_opts, 0);
     rb_define_module_function(Oj, "default_options=", set_def_opts, 1);
