@@ -12,4 +12,6 @@ $: << File.join(File.dirname(__FILE__), "../ext")
 
 require 'oj'
 
-Oj.load_file('josh2.json', :mode => :strict)
+obj = Oj.load_file('bug.json', :mode => :object)
+
+puts Oj.dump(obj, :mode => :object, :indent => 0)
