@@ -40,13 +40,13 @@ class Jam
   end
   alias == eql?
 
-end # Jam
+end# Jam
 
 class Jeez < Jam
   def initialize(x, y)
     super
   end
-  
+
   def to_json()
     %{{"json_class":"#{self.class}","x":#{@x},"y":#{@y}}}
   end
@@ -54,7 +54,7 @@ class Jeez < Jam
   def self.json_create(h)
     self.new(h['x'], h['y'])
   end
-end # Jeez
+end# Jeez
 
 # contributed by sauliusg to fix as_json
 class Orange < Jam
@@ -97,7 +97,7 @@ class Jazz < Jam
   def self.json_create(h)
     self.new(h['x'], h['y'])
   end
-end # Jazz
+end# Jazz
 
 class Range
   def to_hash()
@@ -105,6 +105,20 @@ class Range
   end
 end # Range
 
+# define the symbol
+class ActiveSupport
+end
+
+class RailsLike
+  attr_accessor :x
+  def initialize(x)
+    @x = x
+  end
+  def to_json(options = nil)
+    Oj.dump(self, :mode => :compat)
+  end
+end # RailsLike
+
 class Juice < ::Test::Unit::TestCase
 
   def test0_get_options
@@ -114,12 +128,13 @@ class Juice < ::Test::Unit::TestCase
                    :circular=>false,
                    :auto_define=>false,
                    :symbol_keys=>false,
-                   :ascii_only=>false,
+                   :class_cache=>true,
+                   :escape_mode=>:json,
                    :mode=>:object,
                    :time_format=>:unix,
                    :bigdecimal_as_decimal=>true,
-                   :bigdecimal_load=>false,
-                   :max_stack=>65536,
+                   :bigdecimal_load=>:auto,
+                   :allow_gc=>true,
                    :create_id=>'json_class'}, opts)
   end
 
@@ -130,12 +145,13 @@ class Juice < ::Test::Unit::TestCase
       :circular=>false,
       :auto_define=>false,
       :symbol_keys=>false,
-      :ascii_only=>false,
+      :class_cache=>true,
+      :escape_mode=>:ascii,
       :mode=>:object,
       :time_format=>:unix,
       :bigdecimal_as_decimal=>true,
-      :bigdecimal_load=>false,
-      :max_stack=>65536,
+      :bigdecimal_load=>:auto,
+      :allow_gc=>true,
       :create_id=>'json_class'}
     o2 = {
       :indent=>4,
@@ -143,12 +159,13 @@ class Juice < ::Test::Unit::TestCase
       :circular=>true,
       :auto_define=>true,
       :symbol_keys=>true,
-      :ascii_only=>true,
+      :class_cache=>false,
+      :escape_mode=>:json,
       :mode=>:compat,
       :time_format=>:ruby,
       :bigdecimal_as_decimal=>false,
-      :bigdecimal_load=>true,
-      :max_stack=>4000,
+      :bigdecimal_load=>:bigdecimal,
+      :allow_gc=>false,
       :create_id=>nil}
     o3 = { :indent => 4 }
     Oj.default_options = o2
@@ -180,10 +197,17 @@ class Juice < ::Test::Unit::TestCase
   def test_float
     dump_and_load(0.0, false)
     dump_and_load(12345.6789, false)
+    dump_and_load(70.35, false)
     dump_and_load(-54321.012, false)
     dump_and_load(2.48e16, false)
     dump_and_load(2.48e100 * 1.0e10, false)
     dump_and_load(-2.48e100 * 1.0e10, false)
+    dump_and_load(1/0.0, false)
+    # NaN does not always == NaN
+    json = Oj.dump(0/0.0)
+    assert_equal('NaN', json)
+    loaded = Oj.load(json);
+    assert_equal(true, loaded.nan?)
   end
 
   def test_string
@@ -191,6 +215,7 @@ class Juice < ::Test::Unit::TestCase
     dump_and_load('abc', false)
     dump_and_load("abc\ndef", false)
     dump_and_load("a\u0041", false)
+    assert_equal("a\u0000a", dump_and_load("a\u0000a", false))
   end
 
   def test_string_object
@@ -229,14 +254,45 @@ class Juice < ::Test::Unit::TestCase
     dump_and_load([[nil], 58], false)
   end
 
+  # rails encoding tests
+  def test_does_not_escape_entities_by_default
+    # use Oj to create the hash since some Rubies don't deal nicely with unicode.
+    json = %{{"key":"I <3 this\\u2028space"}}
+    hash = Oj.load(json)
+    out = Oj.dump(hash)
+    assert_equal(json, out)
+  end
+  def test_escapes_entities_by_default_when_configured_to_do_so
+    hash = {'key' => "I <3 this"}
+    Oj.default_options = {:escape_mode => :xss_safe}
+    out = Oj.dump hash
+    assert_equal(%{{"key":"I \\u003c3 this"}}, out)
+  end
+  def test_escapes_entities_when_asked_to
+    hash = {'key' => "I <3 this"}
+    out = Oj.dump(hash, :escape_mode => :xss_safe)
+    assert_equal(%{{"key":"I \\u003c3 this"}}, out)
+  end
+  def test_does_not_escape_entities_when_not_asked_to
+    hash = {'key' => "I <3 this"}
+    out = Oj.dump(hash, :escape_mode => :json)
+    assert_equal(%{{"key":"I <3 this"}}, out)
+  end
+  def test_escapes_common_xss_vectors
+    hash = {'key' => "<script>alert(123) && formatHD()</script>"}
+    out = Oj.dump(hash, :escape_mode => :xss_safe)
+    assert_equal(%{{"key":"\\u003cscript\\u003ealert(123) \\u0026\\u0026 formatHD()\\u003c\\/script\\u003e"}}, out)
+  end
+
   # Symbol
   def test_symbol_strict
     begin
       Oj.dump(:abc, :mode => :strict)
-      assert(false)
     rescue Exception
       assert(true)
+      return
     end
+    assert(false, "*** expected an exception")
   end
   def test_symbol_null
     json = Oj.dump(:abc, :mode => :null)
@@ -245,7 +301,7 @@ class Juice < ::Test::Unit::TestCase
   def test_symbol_compat
     json = Oj.dump(:abc, :mode => :compat)
     assert_equal('"abc"', json)
-  end    
+  end
   def test_symbol_object
     Oj.default_options = { :mode => :object }
     #dump_and_load(''.to_sym, false)
@@ -273,7 +329,7 @@ class Juice < ::Test::Unit::TestCase
     #t = Time.local(2012, 1, 5, 23, 58, 7, 123456)
     json = Oj.dump(t, :mode => :compat)
     assert_equal(%{1325775487.123456000}, json)
-  end    
+  end
   def test_unix_time_compat_precision
     t = Time.xmlschema("2012-01-05T23:58:07.123456789+09:00")
     #t = Time.local(2012, 1, 5, 23, 58, 7, 123456)
@@ -282,23 +338,23 @@ class Juice < ::Test::Unit::TestCase
     t = Time.xmlschema("2012-01-05T23:58:07.999600+09:00")
     json = Oj.dump(t, :mode => :compat, :second_precision => 3)
     assert_equal(%{1325775488.000}, json)
-  end    
+  end
   def test_unix_time_compat_early
     t = Time.xmlschema("1954-01-05T00:00:00.123456789+00:00")
     json = Oj.dump(t, :mode => :compat, :second_precision => 5)
     assert_equal(%{-504575999.87654}, json)
-  end    
+  end
   def test_unix_time_compat_1970
     t = Time.xmlschema("1970-01-01T00:00:00.123456789+00:00")
     json = Oj.dump(t, :mode => :compat, :second_precision => 5)
     assert_equal(%{0.12346}, json)
-  end    
+  end
   def test_ruby_time_compat
     t = Time.xmlschema("2012-01-05T23:58:07.123456000+09:00")
     json = Oj.dump(t, :mode => :compat, :time_format => :ruby)
     #assert_equal(%{"2012-01-05 23:58:07 +0900"}, json)
     assert_equal(%{"#{t.to_s}"}, json)
-  end    
+  end
   def test_xml_time_compat
     begin
       t = Time.new(2012, 1, 5, 23, 58, 7.123456000, 34200)
@@ -317,7 +373,7 @@ class Juice < ::Test::Unit::TestCase
       end
       assert_equal(%{"2012-01-05T23:58:07.123456000%s%02d:%02d"} % [sign, tz / 3600, tz / 60 % 60], json)
     end
-  end    
+  end
   def test_xml_time_compat_no_secs
     begin
       t = Time.new(2012, 1, 5, 23, 58, 7.0, 34200)
@@ -336,7 +392,7 @@ class Juice < ::Test::Unit::TestCase
       end
       assert_equal(%{"2012-01-05T23:58:07%s%02d:%02d"} % [sign, tz / 3600, tz / 60 % 60], json)
     end
-  end    
+  end
   def test_xml_time_compat_precision
     begin
       t = Time.new(2012, 1, 5, 23, 58, 7.123456789, 32400)
@@ -355,12 +411,12 @@ class Juice < ::Test::Unit::TestCase
       end
       assert_equal(%{"2012-01-05T23:58:07.12346%s%02d:%02d"} % [sign, tz / 3600, tz / 60 % 60], json)
     end
-  end    
+  end
   def test_xml_time_compat_precision_round
     begin
       t = Time.new(2012, 1, 5, 23, 58, 7.9996, 32400)
       json = Oj.dump(t, :mode => :compat, :time_format => :xmlschema, :second_precision => 3)
-      assert_equal(%{"2012-01-05T23:58:08.000+09:00"}, json)
+      assert_equal(%{"2012-01-05T23:58:08+09:00"}, json)
     rescue Exception
       # some Rubies (1.8.7) do not allow the timezome to be set
       t = Time.local(2012, 1, 5, 23, 58, 7, 999600)
@@ -374,7 +430,7 @@ class Juice < ::Test::Unit::TestCase
       end
       assert_equal(%{"2012-01-05T23:58:08%s%02d:%02d"} % [sign, tz / 3600, tz / 60 % 60], json)
     end
-  end    
+  end
   def test_xml_time_compat_zulu
     begin
       t = Time.new(2012, 1, 5, 23, 58, 7.0, 0)
@@ -387,7 +443,7 @@ class Juice < ::Test::Unit::TestCase
       #tz = t.utc_offset
       assert_equal(%{"2012-01-05T23:58:07Z"}, json)
     end
-  end    
+  end
   def test_time_object
     t = Time.now()
     Oj.default_options = { :mode => :object }
@@ -415,7 +471,7 @@ class Juice < ::Test::Unit::TestCase
   def test_class_compat
     json = Oj.dump(Juice, :mode => :compat)
     assert_equal(%{"Juice"}, json)
-  end    
+  end
   def test_class_object
     Oj.default_options = { :mode => :object }
     dump_and_load(Juice, false)
@@ -503,11 +559,13 @@ class Juice < ::Test::Unit::TestCase
     assert(%{{
   "^o":"Jeez",
   "x":true,
-  "y":58}} == json ||
+  "y":58
+}} == json ||
 %{{
   "^o":"Jeez",
   "y":58,
-  "x":true}} == json)
+  "x":true
+}} == json)
     obj2 = Oj.load(json, :mode => :object)
     assert_equal(obj, obj2)
   end
@@ -539,11 +597,13 @@ class Juice < ::Test::Unit::TestCase
     assert(%{{
   "^o":"Jazz",
   "x":true,
-  "y":58}} == json ||
+  "y":58
+}} == json ||
 %{{
   "^o":"Jazz",
   "y":58,
-  "x":true}} == json)
+  "x":true
+}} == json)
     obj2 = Oj.load(json, :mode => :object)
     assert_equal(obj, obj2)
   end
@@ -570,12 +630,6 @@ class Juice < ::Test::Unit::TestCase
     obj = Orange.new(true, 58)
     json = Oj.dump(obj, :indent => 2)
     assert(!json.nil?)
-=begin
-    assert_equal(%{{
-  "json_class":"Orange",
-  "x":true,
-  "y":58}}, json)
-=end
     dump_and_load(obj, false)
   end
 
@@ -592,11 +646,13 @@ class Juice < ::Test::Unit::TestCase
     assert(%{{
   "^o":"Orange",
   "x":true,
-  "y":58}} == json ||
+  "y":58
+}} == json ||
 %{{
   "^o":"Orange",
   "y":58,
-  "x":true}} == json)
+  "x":true
+}} == json)
     obj2 = Oj.load(json, :mode => :object)
     assert_equal(obj, obj2)
   end
@@ -621,10 +677,12 @@ class Juice < ::Test::Unit::TestCase
     json = Oj.dump(obj, :mode => :compat, :indent => 2)
     assert(%{{
   "x":true,
-  "y":58}} == json ||
+  "y":58
+}} == json ||
 %{{
   "y":58,
-  "x":true}} == json)
+  "x":true
+}} == json)
   end
   def test_object_object
     obj = Jam.new(true, 58)
@@ -632,15 +690,34 @@ class Juice < ::Test::Unit::TestCase
     assert(%{{
   "^o":"Jam",
   "x":true,
-  "y":58}} == json ||
+  "y":58
+}} == json ||
 %{{
   "^o":"Jam",
   "y":58,
-  "x":true}} == json)
+  "x":true
+}} == json)
     obj2 = Oj.load(json, :mode => :object)
     assert_equal(obj, obj2)
   end
 
+  def test_object_object_no_cache
+    obj = Jam.new(true, 58)
+    json = Oj.dump(obj, :mode => :object, :indent => 2)
+    assert(%{{
+  "^o":"Jam",
+  "x":true,
+  "y":58
+}} == json ||
+%{{
+  "^o":"Jam",
+  "y":58,
+  "x":true
+}} == json)
+    obj2 = Oj.load(json, :mode => :object, :class_cache => false)
+    assert_equal(obj, obj2)
+  end
+
   # Exception
   def test_exception
     err = nil
@@ -692,8 +769,7 @@ class Juice < ::Test::Unit::TestCase
       Oj.default_options = { :mode => :object }
       json = Oj.dump(1..7, :mode => :object, :indent => 0)
       if 'rubinius' == $ruby
-        assert(%{{"^o":"Range","excl":false,"begin":1,"end":7}} == json ||
-               %{{"^o":"Range","begin":1,"end":7,"excl":false}} == json)
+        assert(%{{"^O":"Range","begin":1,"end":7,"exclude_end?":false}} == json)
       elsif 'jruby' == $ruby
         assert(%{{"^O":"Range","begin":1,"end":7,"exclude_end?":false}} == json)
       else
@@ -753,6 +829,13 @@ class Juice < ::Test::Unit::TestCase
     assert_equal(BigDecimal, bg.class)
     assert_equal(orig, bg)
   end
+  def test_float_load
+    orig = BigDecimal.new('80.51')
+    json = Oj.dump(orig, :mode => :compat, :bigdecimal_as_decimal => true)
+    bg = Oj.load(json, :mode => :compat, :bigdecimal_load => :float)
+    assert_equal(Float, bg.class)
+    assert_equal(orig.to_f, bg)
+  end
   def test_bigdecimal_compat_to_json
     orig = BigDecimal.new('80.51')
     BigDecimal.send(:define_method, :to_json) do
@@ -850,12 +933,14 @@ class Juice < ::Test::Unit::TestCase
   "^o":"Jam",
   "^i":1,
   "x":"^r1",
-  "y":58}} == json ||
+  "y":58
+}} == json ||
 %{{
   "^o":"Jam",
   "^i":1,
   "y":58,
-  "x":"^r1"}} == json)
+  "x":"^r1"
+}} == json)
     obj2 = Oj.load(json, :mode => :object, :circular => true)
     assert_equal(obj2.x.__id__, obj2.__id__)
   end
@@ -877,7 +962,8 @@ class Juice < ::Test::Unit::TestCase
     assert_equal(%{[
   "^i1",
   7,
-  "^r1"]}, json)
+  "^r1"
+]}, json)
     a2 = Oj.load(json, :mode => :object, :circular => true)
     assert_equal(a2[1].__id__, a2.__id__)
   end
@@ -896,42 +982,36 @@ class Juice < ::Test::Unit::TestCase
 
 # Stream Deeply Nested
   def test_deep_nest
-    unless 'jruby' == RUBY_DESCRIPTION.split(' ')[0]
-      begin
-        n = 100000
-        Oj.load('[' * n + ']' * n)
-        assert(false)
-      rescue Exception => e
-        assert(e.class == SystemStackError)
-      end
+    #unless 'jruby' == RUBY_DESCRIPTION.split(' ')[0]
+    begin
+      n = 10000
+      Oj.strict_load('[' * n + ']' * n)
+    rescue Exception => e
+      assert(false, e.message)
     end
   end
 
 # Stream IO
   def test_io_string
-    json = %{{
-  "x":true,
-  "y":58,
-  "z": [1,2,3]
-}
-}
-    input = StringIO.new(json)
+    src = { 'x' => true, 'y' => 58, 'z' => [1, 2, 3]}
+    output = StringIO.open("", "w+")
+    Oj.to_stream(output, src)
+
+    input = StringIO.new(output.string())
     obj = Oj.load(input, :mode => :strict)
-    assert_equal({ 'x' => true, 'y' => 58, 'z' => [1, 2, 3]}, obj)
+    assert_equal(src, obj)
   end
 
   def test_io_file
+    src = { 'x' => true, 'y' => 58, 'z' => [1, 2, 3]}
     filename = 'open_file_test.json'
-    File.open(filename, 'w') { |f| f.write(%{{
-  "x":true,
-  "y":58,
-  "z": [1,2,3]
-}
-}) }
+    File.open(filename, "w") { |f|
+      Oj.to_stream(f, src)
+    }
     f = File.new(filename)
     obj = Oj.load(f, :mode => :strict)
     f.close()
-    assert_equal({ 'x' => true, 'y' => 58, 'z' => [1, 2, 3]}, obj)
+    assert_equal(src, obj)
   end
 
 # symbol_keys option
@@ -982,6 +1062,14 @@ class Juice < ::Test::Unit::TestCase
     assert_equal({ 'x' => true, 'y' => 58, 'z' => [1, 2, 3]}, obj)
   end
 
+# Rails re-call test. Active support recalls the json dumper when the to_json
+# method is called. This mimics that and verifies Oj detects it.
+  def test_rails_like
+    obj = RailsLike.new(3)
+    json = Oj.dump(obj, :mode => :compat)
+    assert_equal('{"x":3}', json)
+  end
+
   def dump_and_load(obj, trace=false)
     json = Oj.dump(obj, :indent => 2)
     puts json if trace
